@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -26,10 +26,13 @@
 #include "NdbInfoScanVirtual.hpp"
 
 #include <memory>
+#include <vector>
 
 #include "NdbInfo.hpp"
+#include "my_byteorder.h"
 #include "ndbapi/NdbApi.hpp"
 #include "util/OutputStream.hpp"
+#include "util/cstrbuf.h"
 
 using DictionaryList = NdbDictionary::Dictionary::List;
 
@@ -78,6 +81,8 @@ class VirtualTable {
     void write_string(const char *str);
     void write_number(Uint32 val);
     void write_number64(Uint64 val);
+    void write_null();
+    void write_blob(const void *data, Uint32 len);  // borrow data
 
    private:
     friend class NdbInfoScanVirtual;
@@ -130,9 +135,17 @@ class VirtualTable {
 
   virtual ~VirtualTable() = 0;
 
+  static constexpr int STRING_BUFFER = 516;  // 512 + final null + padding
+
  protected:
   std::map<int, int> m_index;
+  struct free_deleter {
+    void operator()(void *ptr) const { ::free(ptr); }
+  };
+  mutable std::vector<std::unique_ptr<void, free_deleter>> m_pending_frees;
   static constexpr int column_buff_size = 512;
+  static constexpr const char *TRUNCATION_MARK = ",...";
+  static constexpr int TRUNCATE_AT = 508;  // 512 - TRUNCATION_MARK size
 };
 
 /*
@@ -268,6 +281,45 @@ void VirtualTable::Row::write_number64(Uint64 val) {
   DBUG_VOID_RETURN;
 }
 
+void VirtualTable::Row::write_null() {
+  DBUG_ENTER("write_null");
+  DBUG_PRINT("enter", ("%s", "NULL"));
+
+  const unsigned col_idx = m_col_counter++;
+  if (!m_owner->m_recAttrs.is_requested(col_idx)) DBUG_VOID_RETURN;
+
+  // setup RecAttr
+  m_owner->m_recAttrs.set_recattr(col_idx, nullptr, 0);
+
+  DBUG_VOID_RETURN;
+}
+
+void VirtualTable::Row::write_blob(const void *data, Uint32 len) {
+  DBUG_ENTER("write_blob");
+  DBUG_PRINT("enter", ("data: %p, length: %u", data, len));
+
+  assert(check_data_type(NdbInfo::Column::Blob));
+
+  const unsigned col_idx = m_col_counter++;
+  if (!m_owner->m_recAttrs.is_requested(col_idx)) DBUG_VOID_RETURN;
+
+  const size_t clen = 4 + 8;
+  if (!check_buffer_space(clen)) return;
+
+  // setup RecAttr
+  m_owner->m_recAttrs.set_recattr(col_idx, m_curr, clen);
+
+  /*
+   * Write data length and pointer to buffer without making a copy. That is,
+   * caller must ensure data pointer is valid as long as needed.
+   */
+  int4store(m_curr, len);
+  memcpy(m_curr + 4, static_cast<const void *>(&data), 8);
+  m_curr += 12;
+
+  DBUG_VOID_RETURN;
+}
+
 VirtualTable::Row::Row(NdbInfoScanVirtual *owner, const NdbInfo::Table *table,
                        char *buffer, size_t buf_size)
     : m_owner(owner),
@@ -543,7 +595,10 @@ int NdbInfoScanVirtual::execute() {
           buffer_size += sizeof(Uint64);
           break;
         case NdbInfo::Column::String:
-          buffer_size += 516;  // Varchar(512) plus terminator plus padding
+          buffer_size += VirtualTable::STRING_BUFFER;
+          break;
+        case NdbInfo::Column::Blob:
+          buffer_size += 12;
           break;
       }
     }
@@ -1126,6 +1181,13 @@ class BackupIdTable : public VirtualTable {
                           &fragment, &row_id))
       return NdbInfo::ERR_ClusterFailure;
 
+    /* Backup_id is stored in SYSTAB_0 as a 64 bit value,
+     * but only the low 32 bits are used for ids.
+     * Avoid user confusion by returning only the
+     * low 32 bits via NdbInfo.
+     */
+    backup_id &= 0xffffffff;
+
     w.write_number64(backup_id);  // id
     w.write_number(fragment);     // fragment
     w.write_number64(row_id);     // row_id
@@ -1263,6 +1325,27 @@ class IndexStatsTable : public VirtualTable {
   }
 };
 
+/* If a column name contains a dot, backtick, or comma, then quote it in
+   backticks, and escape ` within the name as ``.
+*/
+static BaseString quoteColumnName(BaseString name) {
+  Vector<BaseString> check(2);
+  int split = name.split(check, "`,.", 2);
+  if (likely(split == 1)) return name;
+
+  BaseString result;
+  result.append('`');
+  for (size_t i = 0; i < name.length(); i++) {
+    const char &c = name.c_str()[i];
+    if (c == '`')
+      result.append("``");
+    else
+      result.append(c);
+  }
+  result.append('`');
+  return result;
+}
+
 class DictionaryTablesTable : public VirtualTable {
  public:
   NdbInfo::Table *get_instance() const override {
@@ -1322,7 +1405,11 @@ class DictionaryTablesTable : public VirtualTable {
         !tab->addColumn(
             NdbInfo::Column("GCI_bits", 25, NdbInfo::Column::Number)) ||
         !tab->addColumn(
-            NdbInfo::Column("author_bits", 26, NdbInfo::Column::Number)))
+            NdbInfo::Column("author_bits", 26, NdbInfo::Column::Number)) ||
+        !tab->addColumn(NdbInfo::Column("extra_metadata_version", 27,
+                                        NdbInfo::Column::Number)) ||
+        !tab->addColumn(
+            NdbInfo::Column("extra_metadata", 28, NdbInfo::Column::Blob)))
       return nullptr;
     return tab;
   }
@@ -1348,8 +1435,7 @@ class DictionaryTablesTable : public VirtualTable {
 
     {
       NdbDictionary::HashMap hashMap;
-      char pk_cols[column_buff_size];
-      size_t s = 0;
+      cstrbuf<column_buff_size> pk_cols;
 
       // Get value for column "dynamic"
       int is_dynamic = [t]() {
@@ -1364,42 +1450,59 @@ class DictionaryTablesTable : public VirtualTable {
 
       // Get value for column "primary_key"
       for (int i = 0; i < t->getNoOfPrimaryKeys(); i++) {
-        if (i) s += snprintf(pk_cols + s, column_buff_size - s, ",");
-        s += snprintf(pk_cols + s, column_buff_size - s, "%s",
-                      t->getPrimaryKey(i));
+        BaseString col = quoteColumnName(t->getPrimaryKey(i));
+        if (pk_cols.length() + 1 + col.length() < TRUNCATE_AT) {
+          if (i) pk_cols.append(",");
+          pk_cols.append(col.c_str());
+        } else {
+          pk_cols.append(TRUNCATION_MARK);
+          break;
+        }
       }
 
       // Get tablespace ID
       Uint32 tablespace_id;
       if (!t->getTablespace(&tablespace_id)) tablespace_id = 0;
 
-      w.write_string(elem->database);                      // database
-      w.write_string(t->getName());                        // name
-      w.write_number(t->getTableId());                     // table_id
-      w.write_number(t->getObjectStatus() + 1);            // status
-      w.write_number(t->getNoOfColumns());                 // attributes
-      w.write_number(t->getNoOfPrimaryKeys());             // primary_key_cols
-      w.write_string(pk_cols),                             // primary_key
-          w.write_number((int)(t->getStorageType()) + 1);  // storage
-      w.write_number(t->getLogging());                     // logging
-      w.write_number(is_dynamic);                          // dynamic
-      w.write_number(t->getReadBackupFlag());              // read_backup
-      w.write_number(t->getFullyReplicated());             // fully_replicated
-      w.write_number(t->getRowChecksumIndicator());        // checksum
-      w.write_number(t->getRowSizeInBytes());              // row_size
-      w.write_number64(t->getMinRows());                   // min_rows
-      w.write_number64(t->getMaxRows());                   // max_rows
-      w.write_number(tablespace_id);                       // tablespace
-      w.write_number((int)t->getFragmentType());           // fragment_type
-      w.write_string(hashMap.getName());                   // hash_map
-      w.write_number(t->getFragmentCount());               // fragments
-      w.write_number(t->getPartitionCount());              // partitions
-      w.write_string(t->getPartitionBalanceString());      // partition_balance
-      w.write_number(t->getRowGCIIndicator());             // contains_GCI
-      w.write_number(t->getSingleUserMode() + 1);          // single_user_mode
-      w.write_number(t->getForceVarPart());                // force_var_part
-      w.write_number(t->getExtraRowGciBits());             // GCI_bits
-      w.write_number(t->getExtraRowAuthorBits());          // author_bits
+      w.write_string(elem->database);                  // database
+      w.write_string(t->getName());                    // name
+      w.write_number(t->getTableId());                 // table_id
+      w.write_number(t->getObjectStatus() + 1);        // status
+      w.write_number(t->getNoOfColumns());             // attributes
+      w.write_number(t->getNoOfPrimaryKeys());         // primary_key_cols
+      w.write_string(pk_cols.c_str());                 // primary_key
+      w.write_number((int)(t->getStorageType()) + 1);  // storage
+      w.write_number(t->getLogging());                 // logging
+      w.write_number(is_dynamic);                      // dynamic
+      w.write_number(t->getReadBackupFlag());          // read_backup
+      w.write_number(t->getFullyReplicated());         // fully_replicated
+      w.write_number(t->getRowChecksumIndicator());    // checksum
+      w.write_number(t->getRowSizeInBytes());          // row_size
+      w.write_number64(t->getMinRows());               // min_rows
+      w.write_number64(t->getMaxRows());               // max_rows
+      w.write_number(tablespace_id);                   // tablespace
+      w.write_number((int)t->getFragmentType());       // fragment_type
+      w.write_string(hashMap.getName());               // hash_map
+      w.write_number(t->getFragmentCount());           // fragments
+      w.write_number(t->getPartitionCount());          // partitions
+      w.write_string(t->getPartitionBalanceString());  // partition_balance
+      w.write_number(t->getRowGCIIndicator());         // contains_GCI
+      w.write_number(t->getSingleUserMode() + 1);      // single_user_mode
+      w.write_number(t->getForceVarPart());            // force_var_part
+      w.write_number(t->getExtraRowGciBits());         // GCI_bits
+      w.write_number(t->getExtraRowAuthorBits());      // author_bits
+      void *metadata = nullptr;
+      Uint32 metalen = 0;
+      Uint32 ver = 0;
+      if (t->getExtraMetadata(ver, &metadata, &metalen) == 0) {
+        m_pending_frees.emplace_back(metadata);
+        assert(metadata != nullptr);
+        w.write_number(ver);              // extra_metadata_version
+        w.write_blob(metadata, metalen);  // extra_metadata
+      } else {
+        w.write_null();  // extra_metadata_version
+        w.write_null();  // extra_metadata
+      }
     }
     return 1;
   }
@@ -1510,16 +1613,20 @@ class EventsTable : public VirtualTable {
     } while (event == nullptr);
 
     {
-      char event_columns[column_buff_size];
-      size_t s = 0;
+      cstrbuf<column_buff_size> event_columns;
       Uint32 reporting = 0;
       Uint32 table_event = 0;
 
       /* columns */
       for (int i = 0; i < event->getNoOfEventColumns(); i++) {
-        if (i) s += snprintf(event_columns + s, column_buff_size - s, ",");
-        s += snprintf(event_columns + s, column_buff_size - s, "%s",
-                      event->getEventColumn(i)->getName());
+        BaseString col = quoteColumnName(event->getEventColumn(i)->getName());
+        if (event_columns.length() + 1 + col.length() < TRUNCATE_AT) {
+          if (i) event_columns.append(",");
+          event_columns.append(col.c_str());
+        } else {
+          event_columns.append(TRUNCATION_MARK);
+          break;
+        }
       }
 
       /* reporting
@@ -1550,7 +1657,7 @@ class EventsTable : public VirtualTable {
       w.write_string(elem->name);                        // name
       w.write_number(event->getTable()->getObjectId());  // table_id
       w.write_number(reporting);                         // reporting
-      w.write_string(event_columns);                     // columns
+      w.write_string(event_columns.c_str());             // columns
       w.write_number(table_event);                       // table_event
     }
     return 1;
@@ -1590,17 +1697,21 @@ class IndexColumnsTable : public VirtualTable {
 
   int read_row(VirtualScanContext *ctx, VirtualTable::Row &w,
                Uint32) const override {
-    char index_columns[column_buff_size];
-    size_t s = 0;
+    cstrbuf<column_buff_size> index_columns;
 
     const NdbDictionary::Index *index = ctx->nextIndex();
     if (index == nullptr) return 0;  // end of list
 
     /* columns */
     for (unsigned int i = 0; i < index->getNoOfColumns(); i++) {
-      if (i) s += snprintf(index_columns + s, column_buff_size - s, ",");
-      s += snprintf(index_columns + s, column_buff_size - s, "%s",
-                    index->getColumn(i)->getName());
+      BaseString col = quoteColumnName(index->getColumn(i)->getName());
+      if (index_columns.length() + 1 + col.length() < TRUNCATE_AT) {
+        if (i) index_columns.append(",");
+        index_columns.append(col.c_str());
+      } else {
+        index_columns.append(TRUNCATION_MARK);
+        break;
+      }
     }
 
     w.write_number(ctx->getTable()->getTableId());  // table_id
@@ -1610,7 +1721,7 @@ class IndexColumnsTable : public VirtualTable {
     w.write_string(index->getName());               // index_name
     w.write_number(index->getType());               // index_type
     w.write_number(index->getObjectStatus() + 1);   // status
-    w.write_string(index_columns);                  // columns
+    w.write_string(index_columns.c_str());          // columns
 
     return 1;
   }
@@ -1653,10 +1764,9 @@ class ForeignKeysTable : public VirtualTable {
 
   int read_row(VirtualScanContext *ctx, VirtualTable::Row &w,
                Uint32 row) const override {
-    char parent_columns[column_buff_size];
-    char child_columns[column_buff_size];
+    cstrbuf<column_buff_size> parent_columns;
+    cstrbuf<column_buff_size> child_columns;
     static constexpr const char *pk = "PK";
-    size_t s = 0;
 
     NdbDictionary::ForeignKey fk;
     const DictionaryList::Element *elem;
@@ -1668,15 +1778,14 @@ class ForeignKeysTable : public VirtualTable {
 
     /* parent_columns */
     for (unsigned int i = 0; i < fk.getParentColumnCount(); i++) {
-      s += snprintf(parent_columns + s, column_buff_size - s, "%s%d",
-                    i ? "," : "", fk.getParentColumnNo(i));
+      if (i) parent_columns.append(",");
+      parent_columns.appendf("%d", fk.getParentColumnNo(i));
     }
 
     /* child_columns */
-    s = 0;
     for (unsigned int i = 0; i < fk.getChildColumnCount(); i++) {
-      s += snprintf(child_columns + s, column_buff_size - s, "%s%d",
-                    i ? "," : "", fk.getChildColumnNo(i));
+      if (i) child_columns.append(",");
+      child_columns.appendf("%d", fk.getChildColumnNo(i));
     }
 
     const char *childIdx = fk.getChildIndex() ? fk.getChildIndex() : pk;
@@ -1685,9 +1794,9 @@ class ForeignKeysTable : public VirtualTable {
     w.write_number(fk.getObjectId());            // object_id
     w.write_string(fk.getName());                // name
     w.write_string(fk.getParentTable());         // parent_table
-    w.write_string(parent_columns);              // parent_columns
+    w.write_string(parent_columns.c_str());      // parent_columns
     w.write_string(fk.getChildTable());          // child_table
-    w.write_string(child_columns);               // child_columns
+    w.write_string(child_columns.c_str());       // child_columns
     w.write_string(childIdx);                    // parent_index
     w.write_string(parentIdx);                   // child_index
     w.write_number(fk.getOnUpdateAction() + 1);  // on_update
