@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2004, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2004, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -123,10 +123,10 @@ class BackupRestore : public BackupConsumer {
   virtual void cback(int result, restore_callback_t *cb);
   virtual void cback_logentry(int result, restore_callback_t *cb);
   virtual bool errorHandler(restore_callback_t *cb);
-  void endOfTuples() override;
+  bool endOfTuples() override;
   bool logEntry(const LogEntry &) override;
   void logEntry_a(restore_callback_t *cb);
-  void endOfLogEntrys() override;
+  bool endOfLogEntrys() override;
   bool prepare_staging(const TableS &) override;
   bool finalize_staging(const TableS &) override;
   bool finalize_table(const TableS &) override;
@@ -220,6 +220,7 @@ class BackupRestore : public BackupConsumer {
   void update_next_auto_val(Uint32 orig_table_id, Uint64 next_val);
   bool get_fatal_error();
   void set_fatal_error(bool);
+  void report_error(restore_callback_t *cb, const NdbError &errObj);
 
   Ndb *m_ndb;
   Ndb_cluster_connection *m_cluster_connection;
@@ -256,8 +257,8 @@ class BackupRestore : public BackupConsumer {
   Uint64 m_logBytes;
   Uint64 m_dataBytes;
 
-  Uint32 m_logCount;
-  Uint32 m_dataCount;
+  Uint64 m_logCount;
+  Uint64 m_dataCount;
 
   static const Uint32 INSTANCE_ID_LEN = 20;
   char m_instance_name[INSTANCE_ID_LEN];
